#This source code is able to freely use for Blender and users.

bl_info = {
    "name": "Selection to custom grid",
    "author": "TMO",
    "version": (0, 1, 0),
    "blender": (2, 6, 0),
    "location": "View3D > Object > Snap > Selection to custom grid",
    "description": "Move to intersection",
    "warning": "",
    "wiki_url": "http://www.sousakuba.com/blender/index.html",
    "tracker_url": "",
    "category": "3D View"}

import os
import bpy
from bpy.props import StringProperty
from bpy.props import *

import mathutils
from mathutils import Vector, Matrix

class MSG():
    msg_type = ''
    msg = ''

class MessageBox(bpy.types.Operator, MSG):
    bl_idname = 'va.msg_box'
    bl_label = ''

    def draw(self, context):
        if self.msg_type == 'Error':
            self.t = 'ERROR'
        else:
            self.t = 'NONE'

        layout = self.layout
        row = layout.split(0.25)
        row.label(self.msg_type, icon = self.t)
        row.label(self.msg)

    def execute(self, context):
        return {'FINISHED'}

    def invoke(self, context, event):
        return context.window_manager.invoke_popup(self)

class GridPos:
    #pos = Vector
    #v1 = Vector
    #v2 = Vector
    def __init__(self, pos, v1, v2):
        self.pos = pos
        self.v1 = v1
        self.v2 = v2

def cross(l,r):
    return Vector( ( (l.y * r.z) - (l.z * r.y), (l.z * r.x) - (l.x * r.z), (l.x * r.y) - (l.y * r.x) ) )

def zoom(v, len):
    v.x *= len;
    v.y *= len;
    v.z *= len;

def move(v, ofs):
    v.x += ofs.x;
    v.y += ofs.y;
    v.z += ofs.z;

def vlen(v):
    return pow( v.x*v.x + v.y*v.y + v.z*v.z, 0.5 )

def vlen2(v):
    return v.x*v.x + v.y*v.y + v.z*v.z

def same_pos(v1, v2):
    if vlen2(v1-v2) < 0.001:
        return True;
    return False

def intersect_lines(a, b, c, d ):

    if same_pos(a,c) or same_pos(a,d):
        return a;
    if same_pos(b,c) or same_pos(b,d):
        return b;

    ab = b-a;
    ac = c-a;
    cd = d-c;

    cdab = cross( cd,ab )
    cdac = cross( cd,ac )

    len_cdac = vlen(cdac)
    len_cdab = vlen(cdab)

    #chk parallel
    if len_cdac < 0.001:
        return None
    if len_cdab < 0.001:
        return None


    ret = b-a
    zoom(ret, len_cdac / len_cdab )

    move(ret, a);

    len1 = vlen( a-ret )
    len2 = vlen( b-ret )

    len_ab = vlen(ab)

    if len1 + len2 > len_ab+0.001:
        return None

    len1 = vlen( c-ret )
    len2 = vlen( d-ret )
    
    len_cd = vlen(cd)
    
    if len1 + len2 > len_cd+0.001:
        return None

    return ret

def get_grid(name):

    ob = bpy.data.objects.get(name)

    if ob==None:
        MSG.msg_type = 'Error'
        MSG.msg = "Cannot find the object named \"grid\"."
        bpy.ops.va.msg_box('INVOKE_DEFAULT')
        return None

    me = bpy.data.meshes[ob.data.name]
    if me==None:
        MSG.msg_type = 'Error'
        MSG.msg = "Cannot find mesh in object."
        bpy.ops.va.msg_box('INVOKE_DEFAULT')
        return None

    ob_org = Vector(( ob.location[0], ob.location[1], ob.location[2] ) )

    #verts = []
    #agls1 = []
    #agls2 = []

    gpos = []

    max=len(me.edges)

    for i in range(max):
        for j in range(i+1,max):

            edge1 = me.edges[i];
            edge2 = me.edges[j];

            iv1 = me.vertices[ edge1.vertices[0] ].co + ob_org;
            iv2 = me.vertices[ edge1.vertices[1] ].co + ob_org;

            jv1 = me.vertices[ edge2.vertices[0] ].co + ob_org;
            jv2 = me.vertices[ edge2.vertices[1] ].co + ob_org;

            pos = intersect_lines(iv1, iv2, jv1, jv2)

            if not pos is None:
                #verts.append( pos );
                #print type(me.edges[i].v2.co);
                #agls1.append( me.edges[i].v2.co - me.edges[i].v1.co );
                #agls2.append( me.edges[j].v2.co - me.edges[j].v1.co );

                gpos.append( GridPos(pos, iv2 - iv1, jv2 - jv1 ) );

    if len( gpos ) is 0:
        MSG.msg_type = ''
        MSG.msg = "Cannot find grid intersection."
        bpy.ops.va.msg_box('INVOKE_DEFAULT')
        return None

    return gpos

def SnapEdgeCrossObjectMode(tgtname):

    verts = get_grid(tgtname);

    if verts is None:
        return

    #bpy.ops.mesh.select_all(action='DESELECT')
    #bpy.ops.object.mode_set(mode='OBJECT')

    for obj in bpy.context.selected_objects:

        if obj.name == tgtname:
            continue

        pos = Vector( ( obj.location[0],obj.location[1],obj.location[2] ) )
        near_v = None
        minlen = 0

        for v in verts:

            dist = vlen(v.pos - pos)

            if near_v is None:
                minlen = dist
                near_v = v.pos
                continue

            if dist < minlen:
                minlen = dist
                near_v = v.pos


        obj.location[0] = near_v.x;
        obj.location[1] = near_v.y;
        obj.location[2] = near_v.z;

    #bpy.ops.object.mode_set(mode='EDIT')


def SnapEdgeCrossEditMode(tgtname):

    if bpy.context.active_object.name == tgtname:
        MSG.msg_type = ''
        MSG.msg = "select object is grid"
        bpy.ops.va.msg_box('INVOKE_DEFAULT')
        return

    #need for update select
    bpy.ops.object.editmode_toggle()
    bpy.ops.object.editmode_toggle()

    verts_grid = get_grid(tgtname);

    if verts_grid is None:
        return

    #need for change update
    bpy.ops.object.mode_set(mode='OBJECT')

    obj  = bpy.context.active_object
    mesh = obj.data
    ob_org = Vector(( obj.location[0], obj.location[1], obj.location[2] ) )

    for v in mesh.vertices:

        if not v.select:
            continue

        pos = Vector( ( v.co[0], v.co[1], v.co[2] ) ) + ob_org

        near_v = None
        minlen = 0

        for v_g in verts_grid:

            dist = vlen(v_g.pos - pos)

            if near_v is None:
                minlen = dist
                near_v = v_g.pos
                continue

            if dist < minlen:
                minlen = dist
                near_v = v_g.pos

        v.co[0] = near_v.x - ob_org.x;
        v.co[1] = near_v.y - ob_org.y;
        v.co[2] = near_v.z - ob_org.z;

    bpy.ops.object.mode_set(mode='EDIT')

    return

def SnapEdgeCross():

    tgtname = 'grid'

    if bpy.context.active_object.mode == 'OBJECT':
        SnapEdgeCrossObjectMode(tgtname)
    elif bpy.context.active_object.mode == 'EDIT':
        SnapEdgeCrossEditMode(tgtname)
    else:
        MSG.msg_type = 'Error'
        MSG.msg = "this script run OBJECT or EDIT mode"
        bpy.ops.va.msg_box('INVOKE_DEFAULT')

class main(bpy.types.Operator):
    bl_idname="object.selection_to_custom_grid"
    bl_label="Selection to custom grid"
    bl_options = {'REGISTER', 'UNDO'}

    def draw(self, context):
        return

    def execute(self, context):
        SnapEdgeCross()
        return {'FINISHED'}

def menu_func(self, context):
    self.layout.operator(main.bl_idname, main.bl_label, icon="PLUGIN")

def register():
    bpy.utils.register_module(__name__)

    #bpy.types.VIEW3D_MT_select_edit_mesh.append(menu_func)
    #bpy.types.VIEW3D_MT_object.append(menu_func)
    bpy.types.VIEW3D_MT_snap.append(menu_func)

def unregister():
    bpy.utils.unregister_module(__name__)

    bpy.types.VIEW3D_MT_snap.remove(menu_func)

if __name__ == "__main__":
    register()
